@@ -21,7 +21,7 @@ from numba import njit
 from numba.experimental import jitclass
 from numba import int8, float32
 from quantiphy import Quantity
-from scipy.sparse import dia_matrix
+from scipy.sparse import dia_matrix, csr_matrix
 from scipy import interpolate
 import param
 import networkx as nx
@@ -46,6 +46,17 @@ class InitialConcentrationError(ValueError):
         )
 
 
+class DiffusionCoefficientError(ValueError):
+    def __init__(self, ion):
+        super().__init__(
+            """No diffusion coeficient defined for the ion {}
+               define it using [section].D=dict([ion]=[initial_concentration])
+                    """.format(
+                ion
+            )
+        )
+
+
 class SectionAttribute(param.Range):
     """Parameter than can be a tuple or a value"""
 
@@ -106,21 +117,21 @@ class Neuron(param.Parameterized):
         self._x = None
         self._y = None
         self._graph = nx.Graph()
+        self.__traversal_source__ = None
+        self.__traversal_func__ = nx.dfs_edges
         if sections is not None:
             if issubclass(type(sections), dict):
                 self.add_sections_from_dict(sections)
             else:
                 self.add_sections_from_list(sections)
-
-        self.__traversal_func__ = nx.dfs_edges
-        self.__traversal_source__ = None
+            self.__traversal_source__ = list(self.sections.values())[0][0]
 
     @property
     def graph(self):
         return self._graph
 
     @property
-    def sections(self, traversal=None):
+    def sections(self):
         G = self.graph
         if self.__traversal_func__ is None:
             edges = G.edges
@@ -138,6 +149,8 @@ class Neuron(param.Parameterized):
         if not issubclass(type(sec), Section):
             raise ValueError("Must be a section (sinaps.Section)")
         self.graph.add_edge(i, j, section=sec)
+        if self.__traversal_source__ is None:
+            self.__traversal_source__ = i
 
     def add_sections_from_dict(self, sections):
         """Add multiple sections from dictionnary {sec:(i,j)}."""
@@ -175,11 +188,11 @@ class Neuron(param.Parameterized):
 
         >>> nrn.add_species(Species.Ca)
 
-        Add species `Ca` with initial concentration 1 mMol/L:
+        Add species `Ca` with initial concentration 1 mMol/L and difusion coeficient  1 𝜇𝑚^2/ms:
 
         >>> nrn = Neuron([(0,1)])
-        >>> nrn.add_species(Species.Ca, C0=1)
-        >>> nrn.sections
+        >>> nrn.add_species(Species.Ca, C0=1, D=1)
+
 
         """
         C0 = C0 if C0 is not None else self.DEFAULT_CONCENTRATION
@@ -204,7 +217,7 @@ class Neuron(param.Parameterized):
                     if sp in D:
                         sec.D[sp] = D[sp]
                     else:
-                        raise InitialConcentrationError(sp)  # TODO
+                        raise DiffusionCoefficientError(sp)
 
     def add_reaction(self, left, right, k1, k2):
         """Add chemical reaction to the Neuron.
@@ -288,6 +301,16 @@ class Neuron(param.Parameterized):
             cch[ch_cls] = ch
         return cch
 
+    def _all_Vsource(self):
+        """Return voltage source objects suitable for the simulation."""
+        v_source = [c for s in self.sections for c in s.Vsource]
+        row = np.array([v.idV for v in v_source], int)
+        col = np.ones((1, len(v_source)), int)
+        data = np.ones_like(row)
+        col = np.zeros_like(row)
+        source_mat = csr_matrix((data, (row, col)), (self.nb_comp, 1))
+        return v_source, source_mat
+
     def _capacitance_array(self):
         """Return the menbrane capacitance for each nodes.
 
@@ -599,6 +622,7 @@ class Section(param.Parameterized):
         Section._next_id += 1
         self.channels_c = []
         self.channels_p = []
+        self.Vsource = []
 
     @property
     def _C_m(self):
@@ -636,19 +660,24 @@ class Section(param.Parameterized):
             Quantity(self.L * 1e-6, "m"),
             Quantity(self.a * 1e-6, "m"),
             Quantity(self.C_m * 1e-6, "F/cm²"),
-            Quantity(self.c_m * 1e-12, "F"),
+            Quantity(self.c_m * 1e-12, "F/μm"),
             Quantity(self.R_l, "Ω.cm"),
-            Quantity(self.r_l * 1e-12, "Ω"),
+            Quantity(self.r_l * 1e-12, "Ω/μm"),
             "\n  + ".join([str(c) for c in self.channels_c]),
             "\n  + ".join(["{}:{}".format(c.position, c) for c in self.channels_p]),
         )
 
     def add_channel(self, C, x=None):
-        """Add channel to the section
-        C: type channel
-        position : relative position along the section (between 0 and 1),
-        if None (default) the channels is treated as a continuous density channel
-        giving a surfacic current in nA/μm2
+        """Add a channel to the section
+
+        Parameters
+        ----------
+        C: Channel
+        x: Float, optional
+            relative position along the section (between 0 and 1),
+            if None (default) the channels is treated as a continuous density channel
+            giving a surfacic current in nA/μm2
+
         """
         if not issubclass(type(C), Channel):
             raise ValueError("Must be a channel (sinaps.Channel)")
@@ -670,6 +699,22 @@ class Section(param.Parameterized):
         self.channels_c = []
         self.channels_p = []
 
+    def add_voltage_source(self, C, x=0):
+        """Add a voltage source to the section
+
+        Parameters
+        ----------
+        V: VoltageSource
+        x : Flaot, optional
+            relative position along the section (between 0 and 1),
+
+        """
+        if not issubclass(type(C), VoltageSource):
+            raise ValueError("Must be a channel (sinaps.VoltageSource)")
+        C = deepcopy(C)
+        C.position = x
+        self.Vsource.append(C)
+
     @property
     def channels(self):
         return self.channels_c + self.channels_p
@@ -701,6 +746,8 @@ class Section(param.Parameterized):
         self.xb = np.linspace(0, self.L, n + 1)
         self.idV = idV0 + np.array(range(n), int)
         idS0 = idS00
+        for c in self.Vsource:
+            c.idV = idV0 + int(min(c.position * n, n - 1))
         for c in self.channels_c:
             c.idS = [k * n + idS0 + np.array(range(n), int) for k in range(c.nb_var)]
             c.idV = self.idV
@@ -742,9 +789,8 @@ class Section(param.Parameterized):
         """
         # np.diff(self.xb) = size of compartiment
         for c in self.channels:
-            s0 = c.__S0(len(self.x))
-            for k in range(c.nb_var):
-                S0[c.idS[k]] = s0[k]
+            if c.nb_var:
+                S0[np.hstack(c.idS)] = c.__S0(len(self.x))
 
     def _fill_C0_array(self, C0, ions):
         """Return the initial concentration for each nodes
@@ -778,7 +824,7 @@ class Section(param.Parameterized):
         return np.diff(self.xb) * self._param_array(self.a) * 2 * PI  # [um2]
 
     def _r_l_array(self):
-        """Return the longitunal resistance between each nodes
+        """Return the longitudinal resistance between each nodes
         init_sim(dx) must have been previously called
         """
         # np.diff(self.x) = distance between centers of compartiment
@@ -789,7 +835,7 @@ class Section(param.Parameterized):
         )  # [MΩ]
 
     def _r_l_end(self):
-        """Return the longitunal resistance between the start/end of the section
+        """Return the longitudinal resistance between the start/end of the section
         and the first/last node
         init_sim(dx) must have been previously called
         """
@@ -934,6 +980,13 @@ class _SimuChannel:
                 )
 
 
+class VoltageSource:
+    """Represents a Voltage Source V = f(t)"""
+
+    def __init__(self, f):
+        self.V = njit(f)
+
+
 @jitclass({"charge": int8, "D": float32})
 class _Species:
     """Represents a species."""
